#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "TStoixeiouOuraPlus.h"
#include "OuraPlus.h"

TOuraPlus Received=NULL, UnAnswered=NULL, Called=NULL;
int InitializeLists();

int PopulateLists(FILE *in);
/* Reads call data from in and puts them at the end of the appropriate list. 
   If a list is longer than limitSize delete oldest call. 
   Format of File
   L #number           // #of Calls in list (if list is longer oldest calls are deleted) 
   R dd/mm/yyyy time(sec) telephone         // put into Received Call List 
   U dd/mm/yyyy time(sec) telephone         // put into Unansewred Call List
   C dd/mm/yyyy time(sec) telephone         // put into Called Call List
*/ 

int ShowList(FILE *out, TOuraPlus CallList, int order);
/* Prints to out all data calls, one per line (date, number).
   if order >= 0 oldest to newest, otherwise newest to oldest
*/

int DeleteOrChangeCalls(FILE *in);
/* Reads pairs of telephone numbers from in, finds call with the first and replaces with second 
   Format of File
   A #telephone     #telephone // change
   D #telephone                // delete
*/
int SearchCallList(FILE * in, FILE *out);
/* Reads telephones from in and searches CallList and writes Call Data
   R dd/mm/yyyy     //one per line
   U dd/mm/yyyy     //one per line
   C dd/mm/yyyy     //one per line
*/

int DestroyLists();

int main(int argc, char *argv[])
{
	FILE *finp, *foutp;
	if (InitializeLists()==0)
	    return -1;
	
    finp=fopen("Calls.txt","r");
    if (PopulateLists(finp)==0)
        return -1;
    fclose(finp);
    
    foutp=fopen("to.txt","w");
    fprintf(foutp, "Received calls in ascending order:\n\n");
    ShowList(foutp, Received, 1);
    fprintf(foutp, "\nCalled calls in ascending order:\n\n");
    ShowList(foutp, Called, 1);
    fprintf(foutp, "\nUnAnswered calls in ascending order:\n\n");
    ShowList(foutp, UnAnswered, 1);
    fprintf(foutp, "\nReceived calls in descending order:\n\n");
    ShowList(foutp, Received, -1);
    fprintf(foutp, "\nCalled calls in descending order:\n\n");
    ShowList(foutp, Called, -1);
    fprintf(foutp, "\nUnAnswered calls in descending order:\n\n");
    ShowList(foutp, UnAnswered, -1);
    
	finp=fopen("DelOrChNumber.txt","r");
    DeleteOrChangeCalls(finp);
    fclose(finp);
    
    fprintf(foutp, "\nReceived calls in ascending order after changes:\n\n");
    ShowList(foutp, Received, 1);
    fprintf(foutp, "\nCalled calls in ascending order after changes:\n\n");
    ShowList(foutp, Called, 1);
    fprintf(foutp, "\nUnAnswered calls in ascending order after changes:\n\n");
    ShowList(foutp, UnAnswered, 1);
    fprintf(foutp, "\nReceived calls in descending order after changes:\n\n");
    ShowList(foutp, Received, -1);
    fprintf(foutp, "\nCalled calls in descending order after changes:\n\n");
    ShowList(foutp, Called, -1);
    fprintf(foutp, "\nUnAnswered calls in descending order after changes:\n\n");
    ShowList(foutp, UnAnswered, -1);
    
	finp=fopen("SearchCalls.txt","r");
	fprintf(foutp, "\nSearch results for calls:\n\n");
    SearchCallList(finp, foutp);
    fclose(finp);
    fclose(foutp);

	DestroyLists();

  
  system("PAUSE");
  return 0;
}

int InitializeLists()
{
 OuraPlus_dimiourgia(&Received);
 if(Received==NULL)
	return 0;
 OuraPlus_dimiourgia(&UnAnswered);
 if(UnAnswered==NULL)
    return 0;
 OuraPlus_dimiourgia(&Called);
 if(Called==NULL)
    return 0;
 return 1;
}

int PopulateLists(FILE *in)
{
 char mode;
 int listmax, i;
 TStoixeiouOuraPlus Elem;
 fscanf(in, "%c %d", &mode, &listmax); /*diavazei to arxiko mode kai to arxiko megisto mikos listas*/
 while (fscanf(in, "%c", &mode)!=EOF){  /*pairnei to epomeno mode kathe fora, mexri na ftasei sto telos tou arxeiou*/
    switch(mode)
		{
		case 'M':
            fscanf(in, "%d", &listmax); /*pairnei to neo megisto mikos listas*/
            for ( i=listmax; i<OuraPlus_megethos(Received); i++)    /*an to yparxon mikos ksepernaei to neo megisto exagei analoga*/
                OuraPlus_exagogiArxh(Received);
            for ( i=listmax; i<OuraPlus_megethos(UnAnswered); i++)
                OuraPlus_exagogiArxh(UnAnswered);
            for ( i=listmax; i<OuraPlus_megethos(Called); i++)
                OuraPlus_exagogiArxh(Called);
			break;
		case 'R':
			if (OuraPlus_megethos(Received)==listmax)   /*elegxos an einai gemati i lista me vasi to megisto epitrepto*/
			    OuraPlus_exagogiArxh(Received); /*an einai gemati exagei to paliotero stoixeio na mpei to neo*/
            TSOuraPlus_readValue(in, &Elem);    /*diavazei ta dedomena apo to arxeio in kai ta pernaei sto Elem*/
            if (OuraPlus_eisagogiTelos(Received, Elem)==0)  /*eisagei to stoixeio sto telos tis listas*/
                return 0;   /*an den ginei epityxos i isagogi epistrefei 0*/
			break;
        case 'U':
			if (OuraPlus_megethos(UnAnswered)==listmax)
			    OuraPlus_exagogiArxh(UnAnswered);
            TSOuraPlus_readValue(in, &Elem);
            if (OuraPlus_eisagogiTelos(UnAnswered, Elem)==0)
                return 0;
			break;
        case 'C':
			if (OuraPlus_megethos(Called)==listmax)
			    OuraPlus_exagogiArxh(Called);
            TSOuraPlus_readValue(in, &Elem);
            if (OuraPlus_eisagogiTelos(Called, Elem)==0)
                return 0;
			break;
		}
 }
 return 1;
}

int ShowList(FILE *out, TOuraPlus CallList, int order)
{
 int i;
 TStoixeiouOuraPlus value;
 if (order==1){ /*an theloume se auxousa seira*/
    OuraPlus_prwtos(CallList);  /*paei ston proto*/
    for (i=0; i<OuraPlus_megethos(CallList); i++){  /*oso den exei diavastei oli i lista*/
        OuraPlus_periexomenoTrexon(CallList, &value);   /*pairnei to periexomeno to trexontos*/
        TSOuraPlus_writeValue (out, value); /*typonei sto arxeio out ta dedomena tou*/
        OuraPlus_epomenos(CallList);    /*paei ston epomeno*/
	}
 }
 else{
    OuraPlus_teleytaios(CallList);  /*paei ston teleutaio*/
    for (i=0; i<OuraPlus_megethos(CallList); i++){  /*oso den exei diavastei oli i lista*/
        OuraPlus_periexomenoTrexon(CallList, &value);   /*pairnei to periexomeno to trexontos*/
        TSOuraPlus_writeValue (out, value); /*typonei sto arxeio out ta dedomena tou*/
        OuraPlus_proigoymenos(CallList);    /*paei ston proigoumeno*/
	}
 }
 return 1;
}

int DeleteOrChangeCalls(FILE *in)
{
 char mode, string1[15], string2[15];
 int i;
 TStoixeiouOuraPlus Elem;
 while (fscanf(in, "%c", &mode)!=EOF){/*pairnei to epomeno mode kathe fora, mexri na ftasei sto telos tou arxeiou*/
    switch(mode)
		{
		case 'D':
            fscanf(in, "%s", string1);  /*pairnei ton arithmo pou theloume na diagrapsoume*/
            OuraPlus_prwtos(Received);  /*paei ston proto*/
			for (i=0; i<OuraPlus_megethos(Received); i++){  /*oso den exei diavastei oli i lista*/
				OuraPlus_periexomenoTrexon( Received, &Elem);   /*pairnei to periexomeno to trexontos*/
				if ( strcmp(string1, Elem.number )==0)  /*an o arithmos gia diagrafi isoutai me to number tou trexontos*/
				    OuraPlus_diagrafiTrexon(Received);  /*to diagrafei*/
				OuraPlus_epomenos(Received);    /*paei ston epomeno*/
			}
			OuraPlus_prwtos(UnAnswered);
			for (i=0; i<OuraPlus_megethos(UnAnswered); i++){
				OuraPlus_periexomenoTrexon( UnAnswered, &Elem);
				if ( strcmp(string1, Elem.number )==0)
				    OuraPlus_diagrafiTrexon(UnAnswered);
				OuraPlus_epomenos(UnAnswered);
   			}
			OuraPlus_prwtos(Called);
			for (i=0; i<OuraPlus_megethos(Called); i++){
				OuraPlus_periexomenoTrexon( Called, &Elem);
				if ( strcmp(string1, Elem.number )==0)
				    OuraPlus_diagrafiTrexon(Called);
				OuraPlus_epomenos(Called);
			}
			break;
        case 'A':
            fscanf(in, "%s", string1);  /*pairnei ton arithmo pou theloume na allaxthei*/
            fscanf(in, "%s", string2);  /*kai auton pou theloume na mpei sti thesi tou*/
            OuraPlus_prwtos(Received);  /*paei ston proto*/
			for (i=0; i<OuraPlus_megethos(Received); i++){  /*oso den exei diavastei oli i lista*/
				OuraPlus_periexomenoTrexon( Received, &Elem);   /*pairnei to periexomeno to trexontos*/
				if ( strcmp(string1, Elem.number )==0){ /*an o arithmos gia allagi isoutai me to number tou trexontos*/
					strcpy( Elem.number, string2);  /*allazei ton arithmo*/
					OuraPlus_allagiTrexon( Received, Elem); /*to kataxorei*/
				}
				OuraPlus_epomenos(Received);    /*paei ston epomeno*/
			}
			OuraPlus_prwtos(UnAnswered);
			for (i=0; i<OuraPlus_megethos(UnAnswered); i++){
				OuraPlus_periexomenoTrexon( UnAnswered, &Elem);
				if ( strcmp(string1, Elem.number )==0){
					strcpy( Elem.number, string2);
					OuraPlus_allagiTrexon( UnAnswered, Elem);
				}
				OuraPlus_epomenos(UnAnswered);
   			}
			OuraPlus_prwtos(Called);
			for (i=0; i<OuraPlus_megethos(Called); i++){
				OuraPlus_periexomenoTrexon( Called, &Elem);
				if ( strcmp(string1, Elem.number )==0){
					strcpy( Elem.number, string2);
					OuraPlus_allagiTrexon( Called, Elem);
				}
				OuraPlus_epomenos(Called);
			}
			break;
		}
 }
 return 0;
}

int SearchCallList(FILE *in, FILE *out)
{
 char mode;
 TStoixeiouOuraPlus Elem;
 while (fscanf(in, "%c", &mode)!=EOF){/*pairnei to epomeno mode kathe fora, mexri na ftasei sto telos tou arxeiou*/
	fscanf(in, "%2d %*1c %2d %*1c %4d", &Elem.date.day, &Elem.date.month, &Elem.date.year);/*pairnei tin imerominia pros anazitisi*/
    switch(mode)
		{
		case 'R':
            if (OuraPlus_anazitisi( Received, Elem, TSOuraPlus_iso)==0){/*kalei tin synartisi anazitisis, an epistrepsei miden vrethike*/
                OuraPlus_periexomenoTrexon( Received, &Elem);/*pairnei to periexomeno tou komvou pou vrethike i imeromnia*/
				fprintf(out, "Received call at: "); 
            	TSOuraPlus_writeValue( out, Elem);  /*ektyponei ta stoixeia tou komvou sto arxeio out*/
			}
			break;
        case 'U':
            if (OuraPlus_anazitisi( UnAnswered, Elem, TSOuraPlus_iso)==0){
                OuraPlus_periexomenoTrexon( UnAnswered, &Elem);
                fprintf(out, "Unanswered call at: ");
            	TSOuraPlus_writeValue( out, Elem);
			}
			break;
        case 'C':
            if (OuraPlus_anazitisi( Called, Elem, TSOuraPlus_iso)==0){
                OuraPlus_periexomenoTrexon( Called, &Elem);
                fprintf(out, "Called call at: ");
            	TSOuraPlus_writeValue( out, Elem);
			}
			break;
		}
 }
 return 0;
}

int DestroyLists()
{
 OuraPlus_katastrofi(&Received);
 if(Received!=NULL)
	return 1;
 OuraPlus_katastrofi(&UnAnswered);
 if(UnAnswered!=NULL)
    return 1;
 OuraPlus_katastrofi(&Called);
 if(Called!=NULL)
    return 1;
 return 0;
}
